/**/
#include "IAList.mqh"
/**************************************************************************************
Base list with generic functionality for a family of all possible types of decendants. 
Specific lists for particular data types can substitute/extend its protocol.
This list was not made abstract, to allow its initiation, eg in dictionaries.
/**************************************************************************************/
template<typename T>class AList:public IAList<T>
  {
public:
   /**accessing**/
   T                 Get(int at) {int c=Count(); bool ok=c>0&&at>=0&&at<c; if(ok) {return _items[at];} return NULL;}
   T                 First() {return Get(0);}
   T                 Last() {int c=Count(); return Get(c-1);}
   T                 operator[](int at) {return Get(at);}
   /**adding**/
   bool              Append(T e) {int c=Count(); bool r=IncreaseAt(c); if(r) _items[c]=e; return r;}
   bool              Prepend(T e) {bool r=IncreaseAt(0); if(r) _items[0]=e; return r;}
   bool              Insert(T e,int at) {bool r=IncreaseAt(at); if(r) _items[at]=e; return r;}
   bool              operator+=(T e) {return Append(e);}
   /**removing**/
   bool              Remove(T e) {int f=Find(e); return ReduceAt(f);}
   bool              RemoveAt(int at) {return ReduceAt(at);}
   bool              RemoveLast() {int c=Count(); return ReduceAt(c-1);}
   bool              RemoveFirst() {return ReduceAt(0);}
   bool              RemoveAll() {ArrayFree(_items); return true;}
   bool              operator-=(T e) {return Remove(e);}
   bool              operator~() {return RemoveAll();}
   /**general**/
   int               Count() {return ArraySize(_items);}
   int               Find(T e) {int c=Count(); for(int i=0; i<c; i++) {if(e==_items[i]) {return i;}} return -1;}
   string            Print() {ArrayPrint(_items); return NULL;}
   /**stack interface**/
   bool              Push(T e) {return Append(e);}
   T                 Top() {return Last();}
   T                 Pop() {T top=Top(); RemoveLast(); return top;}
   /**queue interface**/
   bool              Enqueue(T e) {return Append(e);}
   T                 Peek() {return First();}
   T                 Dequeue() {T peek=Peek(); RemoveFirst(); return peek;}
   /**construction,copying**/
                     AList() {}
                     AList(AList&in) {ArrayCopy(_items,in._items); ArrayCopy(_bin,in._bin);}
protected:
   T                 _items[];
   T                 _bin[];
   /**service methods**/
   bool              IncreaseAt(int);
   bool              ReduceAt(int);
  };
/**************************************************************************************/
template<typename T>bool AList::IncreaseAt(int at)
  {
   int c=ArraySize(_items);
   if(at>c||at<0)
     {return false;}
   ArrayResize(_items,c+1);
   for(int i=c; i>=at; i--)
     {int prev=i-1; if(prev>=0) {_items[i]=_items[prev];}}
   return true;
  }
/**************************************************************************************/
template<typename T>bool AList::ReduceAt(int at)
  {
   int c=ArraySize(_items);
   if(at>=c||at<0)
     {return false;}
   for(int i=at; i<c; i++)
     {int next=i+1; if(next<c) {_items[i]=_items[next];}}
   ArrayResize(_items,c-1);
   return true;
  }
/**/
